﻿using Abp.Extensions;
using DataTransmission.Enum;
using Mapster;
using SnapshotWinFormsApp.Application.Dtos;
using SnapshotWinFormsApp.Application.Interfaces;
using SnapshotWinFormsApp.Entities.NewHotline;
using SnapshotWinFormsApp.Entities.OldHotline;
using SnapshotWinFormsApp.Repository;
using SnapshotWinFormsApp.Repository.Enum;
using SnapshotWinFormsApp.Repository.Interfaces;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace SnapshotWinFormsApp.Application;

[Description("导入用户信息")]
public class SnapshotUserInfoApplication : IImportApplication
{
    private readonly IBaseRepository<ThirdAccount> _thirdAccountRepo;
    private readonly IRepository<WeChatUserEntity, int> _weChatRepo;
    private readonly IRepository<WeChatFollowUserEntity, int> _weChatFlowRepo;
    private readonly IBaseRepository<Citizen> _userRepo;
    private readonly IRepository<Flow03_PushContent, int> _pushContentRepo;

    public SnapshotUserInfoApplication(CreateInstanceInDto inDto)
    {
        _thirdAccountRepo = new BaseRepository<ThirdAccount>(inDto);
        _weChatRepo = new Repository<WeChatUserEntity, int>(inDto);
        _weChatFlowRepo = new Repository<WeChatFollowUserEntity, int>(inDto);
        _userRepo = new BaseRepository<Citizen>(inDto);
        _pushContentRepo = new Repository<Flow03_PushContent, int>(inDto);
    }

    public async Task ImportAsync(Action<string> log, CancellationToken token)
    {
        await ImportSnapshotUserInfoAsync(log, token);
        log($"正在查询旧数据...");
        var guiderOldItems = await _pushContentRepo.Queryable()
            .Where(m => m.MemberMobile != null && m.MemberName != null)
            .GroupBy(m => new { m.MemberName, m.MemberMobile })
            .Select(m => new { m.MemberName, m.MemberMobile })
            .ToListAsync(token);

        log($"共查询到{guiderOldItems.Count}条数据");
        for (int i = 0;i < guiderOldItems.Count;i++)
        {
            var item = guiderOldItems[i];
            if (token.IsCancellationRequested)
            {
                log("任务已取消");
                return;
            }
            var userId = await _userRepo.Queryable()
                .Where(m => m.PhoneNumber == item.MemberMobile)
                .Select(m => m.Id)
                .FirstAsync(token);
            if (userId.IsNullOrEmpty())
            {
                var userInfo = new Citizen
                {
                    PhoneNumber = item.MemberMobile.Replace("\t", ""),
                    Name = item.MemberName.Replace("\t", ""),
                };
                userInfo.CitizenType = EReadPackUserType.Guider;
                userInfo.IdentityType = EIdentityType.Citizen;
                userId = await _userRepo.InsertAsync(userInfo, token);
                log($"{i}/{guiderOldItems.Count} 插入用户信息: {userId}, {userInfo.Name}, {userInfo.PhoneNumber}");
            }
        }
    }

    public async Task ImportSnapshotUserInfoAsync(Action<string> log, CancellationToken token)
    {
        log($"正在查询旧数据...");
        var items = await _weChatRepo.Queryable()
            .Where(m => m.WUR_UserType == "ssp")
            .ToListAsync(token);
        log($"共查询到{items.Count}条数据");
        var totalCount = items.Count;
        var index = 0;
        foreach (var item in items)
        {
            index++;
            if (token.IsCancellationRequested)
            {
                log("任务已取消");
                return;
            }
            var userId = await _userRepo.Queryable()
                .Where(m => m.PhoneNumber == item.WUR_PhoneNum)
                .Select(m => m.Id)
                .FirstAsync(token);
            if (userId.IsNullOrEmpty())
            {
                var userInfo = item.Adapt<Citizen>();
                userInfo.CitizenType = EReadPackUserType.Citizen;
                userInfo.IdentityType = EIdentityType.Citizen;
                userId = await _userRepo.InsertAsync(userInfo, token);
                log($"{index}/{totalCount} 插入用户信息: {userId}, {userInfo.Name}, {userInfo.PhoneNumber}");
            }
            var thirdId = await _thirdAccountRepo.Queryable()
                .Where(m => m.OpenId == item.WUR_Openid && m.UnIonId == item.WUR_unionid)
                .Select(m => m.Id)
                .FirstAsync(token);
            if (thirdId.IsNullOrEmpty())
            {
                var thirdAccount = item.Adapt<ThirdAccount>();
                thirdAccount.ExternalId = userId;
                thirdAccount.Id = await _thirdAccountRepo.InsertAsync(thirdAccount, token);
                log($"{index}/{totalCount} 插入第三方账号信息: {thirdAccount.Id}, {thirdAccount.OpenId}, {thirdAccount.UserName}, {thirdAccount.PhoneNumber}, {thirdAccount.ExternalId}");
            }
        }
    }
}
